@@ -1,19 +1,20 @@
 import asyncio
-import multiprocessing
 import traceback
+from multiprocessing import Process
+from threading import Thread
 from typing import Any, Callable, Dict, List, Optional
 
 import cloudpickle
 import redis
-from redis.lock import Lock
 
 from motion.route import Route
 from motion.utils import loadState, logger, saveState
 
 
-class UpdateTask(multiprocessing.Process):
+class BaseUpdateTask:
     def __init__(
         self,
+        task_type: str,
         instance_name: str,
         routes: Dict[str, Route],
         save_state_func: Optional[Callable],
@@ -27,7 +28,8 @@ class UpdateTask(multiprocessing.Process):
         running: Any,
     ):
         super().__init__()
-        self.name = f"UpdateTask-{instance_name}"
+        self.task_type = task_type
+        self.name = f"UpdateTask-{task_type}-{instance_name}"
         self.instance_name = instance_name
         self.save_state_func = save_state_func
         self.load_state_func = load_state_func
@@ -43,16 +45,13 @@ class UpdateTask(multiprocessing.Process):
         self.running = running
         self.daemon = True
 
-    def run(self) -> None:
+    def custom_run(self) -> None:
         redis_con = redis.Redis(
             host=self.redis_host,
             port=self.redis_port,
             password=self.redis_password,
             db=self.redis_db,
         )
-        # Acquire a lock
-        lock_timeout = 300  # Lock timeout in seconds
-        lock = Lock(redis_con, self.instance_name, lock_timeout)
 
         while self.running.value:
             item: Dict[str, Any] = {}
@@ -98,9 +97,8 @@ class UpdateTask(multiprocessing.Process):
                 continue
 
             # Run update op
-            acquired_lock = lock.acquire(blocking=True)
-            if acquired_lock:
-                try:
+            try:
+                with redis_con.lock(f"MOTION_LOCK:{self.instance_name}", timeout=30):
                     old_state = loadState(
                         redis_con,
                         self.instance_name,
@@ -126,14 +124,9 @@ class UpdateTask(multiprocessing.Process):
                             self.instance_name,
                             self.save_state_func,
                         )
-                except Exception:
-                    logger.error(traceback.format_exc())
-                    exception_str = str(traceback.format_exc())
-                finally:
-                    logger.info("Releasing lock.")
-                    lock.release()
-            else:
-                logger.error("Lock not acquired; item lost.")
+            except Exception:
+                logger.error(traceback.format_exc())
+                exception_str = str(traceback.format_exc())
 
             redis_con.publish(
                 self.channel_identifiers[queue_name],
@@ -144,3 +137,31 @@ class UpdateTask(multiprocessing.Process):
                     }
                 ),
             )
+
+
+class UpdateProcess(Process):
+    def __init__(
+        self,
+        **kwargs: Any,
+    ) -> None:
+        super().__init__()
+        self.name = f"UpdateTask-{kwargs.get('instance_name', '')}"
+        self.daemon = True
+        self.but = BaseUpdateTask(task_type="process", **kwargs)
+
+    def run(self) -> None:
+        self.but.custom_run()
+
+
+class UpdateThread(Thread):
+    def __init__(
+        self,
+        **kwargs: Any,
+    ) -> None:
+        super().__init__()
+        self.name = f"UpdateTask-{kwargs.get('instance_name', '')}"
+        self.daemon = True
+        self.but = BaseUpdateTask(task_type="thread", **kwargs)
+
+    def run(self) -> None:
+        self.but.custom_run()
